@@ -11,12 +11,11 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: compiler.h,v 1.4 2002/12/30 21:33:57 david Exp $
+ * $Id: compiler.h,v 1.24 2006-03-08 18:16:08 mfranz Exp $
  *
  **************************************************************************/
 
@@ -121,7 +120,7 @@
 #      define STL_STRSTREAM  <strstream>
 
 #    endif
-#  elif __GNUC__ == 3
+#  elif __GNUC__ >= 3
        // g++-3.0.x
 #      define SG_EXPLICIT_FUNCTION_TMPL_ARGS
 #      define SG_NEED_AUTO_PTR
@@ -149,12 +148,6 @@
 
 #endif // __GNUC__
 
-#if defined( __MINGW32__ )
-#  define bcopy(from, to, n) memcpy(to, from, n)
-#  define FG_MEM_COPY(to,from,n) memcpy(to, from, n)
-#  define snprintf _snprintf
-#endif
-
 /* KAI C++ */
 #if defined(__KCC)
 
@@ -219,7 +212,7 @@
 #  define bcopy(from, to, n) memcpy(to, from, n)
 #  define FG_MEM_COPY(to,from,n) memcpy(to, from, n)
 
-#  if _MSC_VER == 1200  // msvc++ 6.0
+#  if _MSC_VER >= 1200  // msvc++ 6.0 or greater
 #    define SG_NAMESPACES
 #    define SG_HAVE_STD
 #    define SG_HAVE_STD_INCLUDES
@@ -235,6 +228,7 @@
 #    define STL_STRING     <string>
 #    define STL_STRSTREAM  <strstream>
 
+#    define isnan _isnan
 #    define snprintf _snprintf
 
 #    pragma warning(disable: 4786) // identifier was truncated to '255' characters
@@ -307,7 +301,11 @@
 #  define STL_ITERATOR   <iterator>
 #  define STL_FSTREAM    <fstream>
 #  define STL_STDEXCEPT  <stdexcept>
+#if (_COMPILER_VERSION < 740)
+#  define STL_STRING     <irix_string>
+#else
 #  define STL_STRING     <string>
+#endif
 #  define STL_STRSTREAM  <strstream>
 
 #pragma set woff 1001,1012,1014,1116,1155,1172,1174
@@ -315,6 +313,9 @@
 
 #ifdef __cplusplus
 #pragma set woff 1682,3303
+#if (_COMPILER_VERSION >= 740)
+#  pragma set woff 3624
+#endif
 #endif
 
 #  define SG_COMPILER_STR "SGI MipsPro compiler version " SG_STRINGIZE(_COMPILER_VERSION)
@@ -322,7 +323,7 @@
 #endif // Native SGI compilers
 
 
-#if defined ( sun )
+#if defined (__sun)
 #  include <strings.h>
 #  include <memory.h>
 #  if defined ( __cplusplus )
@@ -334,7 +335,9 @@
      extern void *memmove(void *, const void *, size_t);
 #  endif // __cplusplus
 
-#  define SG_COMPILER_STR "Sun compiler version " SG_STRINGIZE(__SUNPRO_CC)
+#  if  !defined( __GNUC__ )
+#   define SG_COMPILER_STR "Sun compiler version " SG_STRINGIZE(__SUNPRO_CC)
+#  endif
 
 #endif // sun
 
@@ -362,6 +365,27 @@
 
 #endif // __ICC
 
+//
+// Platform dependent gl.h and glut.h definitions
+//
+
+#ifdef __APPLE__
+#  define SG_GL_H <OpenGL/gl.h>
+#  define SG_GLX_H <AGL/agl.h>
+#  define SG_GLU_H <OpenGL/glu.h>
+#  define SG_GLEXT_H <OpenGL/glext.h>
+#  define SG_GLUT_H <GLUT/glut.h>
+
+inline int (isnan)(double r) { return !(r <= 0 || r >= 0); }
+#else
+#  define SG_GL_H <GL/gl.h>
+#  define SG_GLX_H <GL/glx.h>
+#  define SG_GLU_H <GL/glu.h>
+#  define SG_GLEXT_H <GL/glext.h>
+#  define SG_GLUT_H <GL/glut.h>
+#endif
+
+
 //
 // No user modifiable definitions beyond here.
 //
@@ -443,3 +467,4 @@ inline const_mem_fun_ref_t<_Ret,_Tp> mem_fun_ref(_Ret (_Tp::*__f)() const)
 #endif // SG_INCOMPLETE_FUNCTIONAL
 
 #endif // _SG_COMPILER_H
+
