@@ -7,8 +7,6 @@
 #include <simgear_config.h>
 #endif
 
-#include <simgear/compiler.h>
-
 #include <string.h>             // for strcmp()
 
 #include <vector>
@@ -19,9 +17,9 @@
 #include <plib/ul.h>
 
 #include <simgear/structure/exception.hxx>
-#include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/props/condition.hxx>
 
 #include "animation.hxx"
 #include "model.hxx"
@@ -29,7 +27,7 @@
 SG_USING_STD(vector);
 SG_USING_STD(set);
 
-
+bool sgUseDisplayList = true;
 
 ////////////////////////////////////////////////////////////////////////
 // Global state
@@ -67,6 +65,23 @@ restore_callback (ssgEntity * entity, int mask)
     return 1;
 }
 
+/**
+ * Callback and userData class for conditional rendering.
+ */
+class SGConditionalRender : public ssgBase {
+public:
+  SGConditionalRender(SGCondition *c) : _condition(c) {}
+  bool test() { return _condition->test(); }
+private:
+  SGCondition *_condition;
+};
+
+static int
+model_condition_callback (ssgEntity * entity, int mask)
+{
+  return ((SGConditionalRender *)entity->getUserData())->test();
+}
+
 
 /**
  * Locate a named SSG node in a branch.
@@ -125,6 +140,7 @@ sgMakeAnimation( ssgBranch * model,
                  SGPropertyNode *prop_root,
                  SGPropertyNode_ptr node,
                  double sim_time_sec,
+                 const SGPath &texture_path,
                  set<ssgBranch *> &ignore_branches )
 {
   bool ignore = false;
@@ -159,10 +175,16 @@ sgMakeAnimation( ssgBranch * model,
     ignore = true;
   } else if (!strcmp("alpha-test", type)) {
     animation = new SGAlphaTestAnimation(node);
+  } else if (!strcmp("material", type)) {
+    animation = new SGMaterialAnimation(prop_root, node, texture_path);
   } else if (!strcmp("flash", type)) {
     animation = new SGFlashAnimation(node);
   } else if (!strcmp("dist-scale", type)) {
     animation = new SGDistScaleAnimation(node);
+  } else if (!strcmp("noshadow", type)) {
+    animation = new SGShadowAnimation(prop_root, node);
+  } else if (!strcmp("shader", type)) {
+    animation = new SGShaderAnimation(prop_root, node);
   } else {
     animation = new SGNullAnimation(node);
     SG_LOG(SG_INPUT, SG_WARN, "Unknown animation type " << type);
@@ -222,6 +244,7 @@ static void makeDList( ssgBranch *b, const set<ssgBranch *> &ignore )
   for (int i = 0; i<nb; i++) {
     ssgEntity *e = b->getKid(i);
     if (e->isAKindOf(ssgTypeLeaf())) {
+     if( ((ssgLeaf*)e)->getNumVertices() > 0)
       ((ssgLeaf*)e)->makeDList();
     } else if (e->isAKindOf(ssgTypeBranch()) && ignore.find((ssgBranch *)e) == ignore.end()) {
       makeDList( (ssgBranch*)e, ignore );
@@ -229,6 +252,13 @@ static void makeDList( ssgBranch *b, const set<ssgBranch *> &ignore )
   }
 }
 
+class SGLoaderOptions : public ssgLoaderOptions {
+public:
+  SGLoaderOptions() { ssgSetCurrentOptions( this ); } // Install our own loader options at startup
+  void endLoad() {} // Avoid clearing the texture cache after every model load
+};
+
+static SGLoaderOptions loaderOptions;
 
 
 ////////////////////////////////////////////////////////////////////////
@@ -238,7 +268,8 @@ static void makeDList( ssgBranch *b, const set<ssgBranch *> &ignore )
 ssgBranch *
 sgLoad3DModel( const string &fg_root, const string &path,
                SGPropertyNode *prop_root,
-               double sim_time_sec, ssgEntity *(*load_panel)(SGPropertyNode *) )
+               double sim_time_sec, ssgEntity *(*load_panel)(SGPropertyNode *),
+               SGModelData *data )
 {
   ssgBranch * model = 0;
   SGPropertyNode props;
@@ -267,6 +298,11 @@ sgLoad3DModel( const string &fg_root, const string &path,
     }
   }
 
+                                // Return an empty node for OSG models
+  if (modelpath.str().substr(modelpath.str().size() - 4, 4) == ".osg") {
+    SG_LOG(SG_INPUT, SG_WARN, "Ignoring OSG model " << modelpath.str());
+    return new ssgBranch;
+  }
                                 // Assume that textures are in
                                 // the same location as the XML file.
   if (model == 0) {
@@ -276,7 +312,8 @@ sgLoad3DModel( const string &fg_root, const string &path,
     ssgTexturePath((char *)texturepath.c_str());
     model = (ssgBranch *)ssgLoad((char *)modelpath.c_str());
     if (model == 0)
-      throw sg_exception("Failed to load 3D model");
+      throw sg_io_exception("Failed to load 3D model", 
+			  sg_location(modelpath.str()));
   }
                                 // Set up the alignment node
   ssgTransform * alignmainmodel = new ssgTransform;
@@ -299,6 +336,8 @@ sgLoad3DModel( const string &fg_root, const string &path,
   vector<SGPropertyNode_ptr> model_nodes = props.getChildren("model");
   for (i = 0; i < model_nodes.size(); i++) {
     SGPropertyNode_ptr node = model_nodes[i];
+    const char * submodel = node->getStringValue("path");
+
     ssgTransform * align = new ssgTransform;
     sgMat4 res_matrix;
     sgMakeOffsetsMatrix(&res_matrix,
@@ -310,29 +349,27 @@ sgLoad3DModel( const string &fg_root, const string &path,
                         node->getFloatValue("offsets/z-m", 0.0));
     align->setTransform(res_matrix);
 
-    ssgBranch * kid = sgLoad3DModel( fg_root, node->getStringValue("path"),
-                                     prop_root, sim_time_sec, load_panel );
+    ssgBranch * kid;
+    try {
+      kid = sgLoad3DModel( fg_root, submodel, prop_root, sim_time_sec, load_panel );
+
+    } catch (const sg_throwable &t) {
+      SG_LOG(SG_INPUT, SG_ALERT, "Failed to load submodel: " << t.getFormattedMessage());
+      throw;
+    }
+
+    SGPropertyNode *cond = node->getNode("condition", false);
+    if (cond) {
+      align->setUserData(new SGConditionalRender(sgReadCondition(prop_root, cond)));
+      align->setTravCallback(SSG_CALLBACK_PRETRAV, model_condition_callback);
+    }
+
     align->addKid(kid);
     align->setName(node->getStringValue("name", ""));
     model->addKid(align);
   }
-
-                                // Load animations
-  set<ssgBranch *> ignore_branches;
-  vector<SGPropertyNode_ptr> animation_nodes = props.getChildren("animation");
-  for (i = 0; i < animation_nodes.size(); i++) {
-    const char * name = animation_nodes[i]->getStringValue("name", 0);
-    vector<SGPropertyNode_ptr> name_nodes =
-      animation_nodes[i]->getChildren("object-name");
-    sgMakeAnimation( model, name, name_nodes, prop_root, animation_nodes[i],
-                     sim_time_sec, ignore_branches);
-  }
-
-#if PLIB_VERSION > 183
-  if ( model != 0 ) {
-     makeDList( model, ignore_branches );
-  }
-#endif
+  // reset texture path in case a sub-model has changed it
+  ssgTexturePath((char *)texturepath.c_str());
 
   if ( load_panel ) {
                                 // Load panels
@@ -346,6 +383,31 @@ sgLoad3DModel( const string &fg_root, const string &path,
     }
   }
 
+  if (data) {
+    alignmainmodel->setUserData(data);
+    data->modelLoaded(path, &props, alignmainmodel);
+  }
+                                // Load animations
+  set<ssgBranch *> ignore_branches;
+  vector<SGPropertyNode_ptr> animation_nodes = props.getChildren("animation");
+  for (i = 0; i < animation_nodes.size(); i++) {
+    const char * name = animation_nodes[i]->getStringValue("name", 0);
+    vector<SGPropertyNode_ptr> name_nodes =
+      animation_nodes[i]->getChildren("object-name");
+    sgMakeAnimation( model, name, name_nodes, prop_root, animation_nodes[i],
+                     sim_time_sec, texturepath, ignore_branches);
+  }
+
+#if PLIB_VERSION > 183
+  if ( model != 0 && sgUseDisplayList ) {
+     makeDList( model, ignore_branches );
+  }
+#endif
+
+  int m = props.getIntValue("dump", 0);
+  if (m > 0)
+    model->print(stderr, const_cast<char *>(""), m - 1);
+
   return alignmainmodel;
 }
 
@@ -357,5 +419,10 @@ sgSetModelFilter( bool filter )
   return old;
 }
 
+bool 
+sgCheckAnimationBranch (ssgEntity * entity)
+{
+    return entity->getTravCallback(SSG_CALLBACK_PRETRAV) == animation_callback;
+}
 
 // end of model.cxx
