@@ -3,7 +3,7 @@
 // Written by Curtis Olson, started December 1997.
 // SSG-ified by Curtis Olson, February 2000.
 //
-// Copyright (C) 1997-2000  Curtis L. Olson  - curt@flightgear.org
+// Copyright (C) 1997-2000  Curtis L. Olson  - http://www.flightgear.org/~curt
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -15,12 +15,11 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Library General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: dome.cxx,v 1.5 2003/05/21 17:17:50 ehofman Exp $
+// $Id: dome.cxx,v 1.13 2006-03-08 18:16:09 mfranz Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -33,7 +32,9 @@
 
 #include <math.h>
 
-#include GLUT_H
+#include <simgear/compiler.h>
+
+#include SG_GL_H
 
 #include <plib/sg.h>
 
@@ -47,20 +48,20 @@
 #endif
 
 
-// in meters of course
-static const float center_elev = 0.3125;
+// proportions of max dimensions fed to the build() routine
+static const float center_elev = 1.0;
 
-static const float upper_radius = 0.6250;
-static const float upper_elev = 0.2500;
+static const float upper_radius = 0.6;
+static const float upper_elev = 0.15;
 
-static const float middle_radius = 0.8750;
-static const float middle_elev = 0.1000;
+static const float middle_radius = 0.9;
+static const float middle_elev = 0.08;
 
-static const float lower_radius = 0.8750;
-static const float lower_elev = 0.0000;
+static const float lower_radius = 1.0;
+static const float lower_elev = 0.0;
 
-static const float bottom_radius = 0.6250;
-static const float bottom_elev = -0.0250;
+static const float bottom_radius = 0.8;
+static const float bottom_elev = -0.1;
 
 
 // Set up dome rendering callbacks
@@ -93,6 +94,7 @@ static int sgSkyDomePostDraw( ssgEntity *e ) {
 
 // Constructor
 SGSkyDome::SGSkyDome( void ) {
+    asl = 0.0f;
 }
 
 
@@ -105,7 +107,7 @@ SGSkyDome::~SGSkyDome( void ) {
 ssgBranch * SGSkyDome::build( double hscale, double vscale ) {
     sgVec4 color;
 
-    float theta;
+    double theta;
     int i;
 
     // set up the state
@@ -153,20 +155,20 @@ ssgBranch * SGSkyDome::build( double hscale, double vscale ) {
 		   cos(theta) * upper_radius * hscale,
 		   sin(theta) * upper_radius * hscale,
 		   upper_elev * vscale );
-	
+
 	sgSetVec3( middle_vertex[i],
-		   cos((double)theta) * middle_radius * hscale,
-		   sin((double)theta) * middle_radius * hscale,
+		   cos(theta) * middle_radius * hscale,
+		   sin(theta) * middle_radius * hscale,
 		   middle_elev * vscale );
 
 	sgSetVec3( lower_vertex[i],
-		   cos((double)theta) * lower_radius * hscale,
-		   sin((double)theta) * lower_radius * hscale,
+		   cos(theta) * lower_radius * hscale,
+		   sin(theta) * lower_radius * hscale,
 		   lower_elev * vscale );
 
 	sgSetVec3( bottom_vertex[i],
-		   cos((double)theta) * bottom_radius * hscale,
-		   sin((double)theta) * bottom_radius * hscale,
+		   cos(theta) * bottom_radius * hscale,
+		   sin(theta) * bottom_radius * hscale,
 		   bottom_elev * vscale );
     }
 
@@ -271,6 +273,13 @@ ssgBranch * SGSkyDome::build( double hscale, double vscale ) {
     return dome_transform;
 }
 
+static void fade_to_black( sgVec4 sky_color[], float asl, int count) {
+    const float ref_asl = 10000.0f;
+    const sgVec3 space_color = {0.0f, 0.0f, 0.0f};
+    float d = exp( - asl / ref_asl );
+    for(int i = 0; i < count ; i++)
+        sgLerpVec3( sky_color[i], sky_color[i], space_color, 1.0 - d);
+}
 
 // repaint the sky colors based on current value of sun_angle, sky,
 // and fog colors.  This updates the color arrays for ssgVtxTable.
@@ -332,33 +341,37 @@ bool SGSkyDome::repaint( sgVec4 sky_color, sgVec4 fog_color, double sun_angle,
     sgVec4 lower_color[12];
     sgVec4 bottom_color[12];
 
-    double vis_factor;
+    double vis_factor, cvf = vis;
 
-    if ( vis < 3000.0 ) {
-	vis_factor = (vis - 1000.0) / 2000.0;
-	if ( vis_factor < 0.0 ) {
-	    vis_factor = 0.0;
-	}
-    } else {
-	vis_factor = 1.0;
+    if (cvf > 45000)
+        cvf = 45000;
+
+    vis_factor = (vis - 1000.0) / 2000.0;
+    if ( vis_factor < 0.0 ) {
+        vis_factor = 0.0;
+    } else if ( vis_factor > 1.0) {
+        vis_factor = 1.0;
     }
 
     for ( j = 0; j < 3; j++ ) {
 	diff = sky_color[j] - fog_color[j];
-	center_color[j] = sky_color[j] - diff * ( 1.0 - vis_factor );
+	center_color[j] = sky_color[j]; // - diff * ( 1.0 - vis_factor );
     }
     center_color[3] = 1.0;
 
     for ( i = 0; i < 6; i++ ) {
 	for ( j = 0; j < 3; j++ ) {
-	    diff = sky_color[j] - fog_color[j];
+            double saif = sun_angle/SG_PI;
+	    diff = (sky_color[j] - fog_color[j]) * (0.8 + j * 0.2) * (0.8 + saif - ((6-i)/10));
 
 	    // printf("sky = %.2f  fog = %.2f  diff = %.2f\n", 
 	    //        l->sky_color[j], l->fog_color[j], diff);
 
-	    upper_color[i][j] = sky_color[j] - diff * ( 1.0 - vis_factor * 0.7);
-	    middle_color[i][j] = sky_color[j] - diff * ( 1.0 - vis_factor * 0.1)
-		+ middle_amt[j];
+	    upper_color[i][j] = sky_color[j] - diff *
+                                 ( 1.0 - vis_factor * (0.7 + 0.3 * cvf/45000) );
+	    middle_color[i][j] = sky_color[j] - diff *
+                                 ( 1.0 - vis_factor * (0.1 + 0.85 * cvf/45000) )
+		                 + middle_amt[j];
 	    lower_color[i][j] = fog_color[j] + outer_amt[j];
 
 	    if ( upper_color[i][j] > 1.0 ) { upper_color[i][j] = 1.0; }
@@ -392,14 +405,17 @@ bool SGSkyDome::repaint( sgVec4 sky_color, sgVec4 fog_color, double sun_angle,
 
     for ( i = 6; i < 12; i++ ) {
 	for ( j = 0; j < 3; j++ ) {
-	    diff = sky_color[j] - fog_color[j];
+            double saif = sun_angle/SG_PI;
+            diff = (sky_color[j] - fog_color[j]) * (0.8 + j * 0.2) * (0.8 + saif - ((-i+12)/10));
 
 	    // printf("sky = %.2f  fog = %.2f  diff = %.2f\n", 
 	    //        sky_color[j], fog_color[j], diff);
 
-	    upper_color[i][j] = sky_color[j] - diff * ( 1.0 - vis_factor * 0.7);
-	    middle_color[i][j] = sky_color[j] - diff * ( 1.0 - vis_factor * 0.1)
-		+ middle_amt[j];
+	    upper_color[i][j] = sky_color[j] - diff *
+                                 ( 1.0 - vis_factor * (0.7 + 0.3 * cvf/45000) );
+	    middle_color[i][j] = sky_color[j] - diff *
+                                 ( 1.0 - vis_factor * (0.1 + 0.85 * cvf/45000) )
+                                  + middle_amt[j];
 	    lower_color[i][j] = fog_color[j] + outer_amt[j];
 
 	    if ( upper_color[i][j] > 1.0 ) { upper_color[i][j] = 1.0; }
@@ -428,6 +444,11 @@ bool SGSkyDome::repaint( sgVec4 sky_color, sgVec4 fog_color, double sun_angle,
  	*/
    }
 
+    fade_to_black( (sgVec4 *) center_color, asl * center_elev, 1);
+    fade_to_black( upper_color, (asl+0.05f) * upper_elev, 12);
+    fade_to_black( middle_color, (asl+0.05f) * middle_elev, 12);
+    fade_to_black( lower_color, (asl+0.05f) * lower_elev, 12);
+
     for ( i = 0; i < 12; i++ ) {
 	sgCopyVec4( bottom_color[i], fog_color );
     }
@@ -546,6 +567,6 @@ bool SGSkyDome::reposition( sgVec3 p, double lon, double lat, double spin ) {
     sgSetCoord( &skypos, TRANSFORM );
 
     dome_transform->setTransform( &skypos );
-
+    asl = - skypos.xyz[2];
     return true;
 }
