@@ -2,7 +2,7 @@
 //
 // Written by Curtis Olson, started August 1997.
 //
-// Copyright (C) 1997  Curtis L. Olson  - curt@infoplane.com
+// Copyright (C) 1997  Curtis L. Olson  - http://www.flightgear.org/~curt
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -14,12 +14,11 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Library General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_time.cxx,v 1.4 2003/06/03 18:32:54 curt Exp $
+// $Id: sg_time.cxx,v 1.10.2.2 2007-10-13 13:51:52 durk Exp $
 
 
 #ifdef HAVE_CONFIG_H
@@ -71,7 +70,7 @@ static const double J2000   = 2451545.0 - MJD0;
 static const double SIDRATE = 0.9972695677;
 
 
-void SGTime::init( double lon, double lat,
+void SGTime::init( double lon_rad, double lat_rad,
                    const string& root, time_t init_time )
 {
     SG_LOG( SG_EVENT, SG_INFO, "Initializing Time" );
@@ -96,7 +95,7 @@ void SGTime::init( double lon, double lat,
                 << zone.str() );
         tzContainer = new SGTimeZoneContainer( zone.c_str() );
 
-        SGGeoCoord location( SGD_RADIANS_TO_DEGREES * lat, SGD_RADIANS_TO_DEGREES * lon );
+        SGGeoCoord location( SGD_RADIANS_TO_DEGREES * lat_rad, SGD_RADIANS_TO_DEGREES * lon_rad );
         SGGeoCoord* nearestTz = tzContainer->getNearest(location);
 
         SGPath name( root );
@@ -110,9 +109,10 @@ void SGTime::init( double lon, double lat,
     }
 }
 
-SGTime::SGTime( double lon, double lat, const string& root, time_t init_time )
+SGTime::SGTime( double lon_rad, double lat_rad, const string& root,
+                time_t init_time )
 {
-    init( lon, lat, root, init_time );
+    init( lon_rad, lat_rad, root, init_time );
 }
 
 
@@ -169,7 +169,7 @@ static double sidereal_course( time_t cur_time, const struct tm *gmt, double lng
     start_gmt = sgTimeGetGMT(gmt->tm_year, 2, 21, 12, 0, 0);
   
     SG_LOG( SG_EVENT, SG_DEBUG, "  COURSE: GMT = "
-	    << sgTimeFormatTime(gmt, tbuf) );
+	    << sgTimeFormatTime(gmt, tbuf, 64) );
     SG_LOG( SG_EVENT, SG_DEBUG, "  March 21 noon (GMT) = " << start_gmt );
   
     diff = (now - start_gmt) / (3600.0 * 24.0);
@@ -196,7 +196,9 @@ static double sidereal_course( time_t cur_time, const struct tm *gmt, double lng
 
 
 // Update the time related variables
-void SGTime::update( double lon, double lat, time_t ct, long int warp ) {
+void SGTime::update( double lon_rad, double lat_rad,
+                     time_t ct, long int warp )
+{
     double gst_precise, gst_course;
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
@@ -251,46 +253,51 @@ void SGTime::update( double lon, double lat, time_t ct, long int warp ) {
       
 	gst_diff = gst_precise - gst_course;
 
-	lst = sidereal_course( cur_time, gmt, -(lon * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
+	lst = sidereal_course( cur_time, gmt,
+                               -(lon_rad * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
     } else {
 	// course + difference should drift off very slowly
 	gst = sidereal_course( cur_time, gmt, 0.00 ) + gst_diff;
-	lst = sidereal_course( cur_time, gmt, -(lon * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
+	lst = sidereal_course( cur_time, gmt,
+                               -(lon_rad * SGD_RADIANS_TO_DEGREES) ) + gst_diff;
     }
 
     SG_LOG( SG_EVENT, SG_DEBUG,
 	    "  Current lon=0.00 Sidereal Time = " << gst );
     SG_LOG( SG_EVENT, SG_DEBUG,
 	    "  Current LOCAL Sidereal Time = " << lst << " (" 
-	    << sidereal_precise( mjd, -(lon * SGD_RADIANS_TO_DEGREES) ) 
+	    << sidereal_precise( mjd, -(lon_rad * SGD_RADIANS_TO_DEGREES) ) 
 	    << ") (diff = " << gst_diff << ")" );
 }
 
 
 // Given lon/lat, update timezone information and local_offset
-void SGTime::updateLocal( double lon, double lat, const string& root ) {
+void SGTime::updateLocal( double lon_rad, double lat_rad, const string& root ) {
     // sanity checking
-    if ( lon < -SGD_PI || lon > SGD_PI ) {
+    if ( lon_rad < -SGD_PI || lon_rad> SGD_PI ) {
         // not within -180 ... 180
-        lon = 0.0;
+        lon_rad = 0.0;
     }
-    if ( lat < -SGD_PI * 0.5 || lat > SGD_PI * 0.5 ) {
+    if ( lat_rad < -SGD_PI_2 || lat_rad > SGD_PI_2 ) {
         // not within -90 ... 90
-        lat = 0.0;
+        lat_rad = 0.0;
     }
-    if ( lon != lon ) {
-        // only true if lon == nan
-        SG_LOG( SG_EVENT, SG_ALERT, "  Detected lon == nan, resetting to 0.0" );
-        lon = 0.0;
+    if ( lon_rad != lon_rad ) {
+        // only true if lon_rad == nan
+        SG_LOG( SG_EVENT, SG_ALERT,
+                "  Detected lon_rad == nan, resetting to 0.0" );
+        lon_rad = 0.0;
     }
-    if ( lat != lat ) {
-        // only true if lat == nan
-        SG_LOG( SG_EVENT, SG_ALERT, "  Detected lat == nan, resetting to 0.0" );
-        lat = 0.0;
+    if ( lat_rad != lat_rad ) {
+        // only true if lat_rad == nan
+        SG_LOG( SG_EVENT, SG_ALERT,
+                "  Detected lat_rad == nan, resetting to 0.0" );
+        lat_rad = 0.0;
     }
     time_t currGMT;
     time_t aircraftLocalTime;
-    SGGeoCoord location( SGD_RADIANS_TO_DEGREES * lat, SGD_RADIANS_TO_DEGREES * lon );
+    SGGeoCoord location( SGD_RADIANS_TO_DEGREES * lat_rad,
+                         SGD_RADIANS_TO_DEGREES * lon_rad );
     SGGeoCoord* nearestTz = tzContainer->getNearest(location);
     SGPath zone( root );
     zone.append ( nearestTz->getDescription() );
@@ -522,7 +529,7 @@ time_t sgTimeGetGMT(int year, int month, int day, int hour, int min, int sec)
 #else // ! defined ( MK_TIME_IS_GMT )
 
     // timezone seems to work as a proper offset for Linux & Solaris
-#   if defined( __linux__ ) || defined( __sun__ ) ||defined(__CYGWIN__)
+#   if defined( __linux__ ) || defined(__sun) ||defined(__CYGWIN__)
 #       define TIMEZONE_OFFSET_WORKS 1
 #   endif
 
@@ -573,10 +580,12 @@ time_t sgTimeGetGMT(int year, int month, int day, int hour, int min, int sec)
 
 
 // format time
-char* sgTimeFormatTime( const struct tm* p, char* buf )
+char* sgTimeFormatTime( const struct tm* p, char* buf , int bufSize)
 {
-    sprintf( buf, "%d/%d/%2d %d:%02d:%02d", 
+    if (snprintf( buf, bufSize, "%d/%d/%2d %d:%02d:%02d", 
 	     p->tm_mon, p->tm_mday, p->tm_year,
-	     p->tm_hour, p->tm_min, p->tm_sec);
+	     p->tm_hour, p->tm_min, p->tm_sec) >= bufSize) {
+        SG_LOG(SG_EVENT, SG_ALERT, "Caught overflow in sgTimeFormatTime");
+    } 
     return buf;
 }
